#pragma once

#include "wimp_basics.h"
#include "wimp_icons.h"
#include "wimp_messages.h"

#ifdef __cplusplus
extern "C"
{
#endif

  typedef struct
  {
    unsigned wfOldHasTitlebar : 1;
    unsigned wfIsMoveable : 1;
    unsigned wfOldHasVerticalScrollbar : 1;
    unsigned wfOldHasHorizontalScrollbar : 1;
    unsigned wfWimpRedrawn : 1;
    unsigned wfIsPane : 1;
    unsigned wfCanOpenOffscreen : 1;
    unsigned wfOldNoBackOrClose : 1;
    unsigned wfScrollRequest : 1;
    unsigned wfScrollRequestNoAutorepeat : 1;
    unsigned wfWindowColoursAreGcol : 1;
    unsigned wfAlwaysOnBottom : 1;
    unsigned wfGenerateEvents : 1;
    unsigned wfForceOnscreen : 1;
    unsigned wfIgnoreRightExtent : 1;
    unsigned wfIgnoreLowerExtent : 1;
    unsigned wfIsOpen : 1;
    unsigned wfIsUncovered : 1;
    unsigned wfIsFullsize : 1;
    unsigned wfOpenCausedByToggleSize : 1;
    unsigned wfHasFocus : 1;
    unsigned wfForceToScreen : 1;
    unsigned : 2;
    unsigned wfHasBackIcon : 1;
    unsigned wfHasCloseIcon : 1;
    unsigned wfHasTitlebar : 1;
    unsigned wfHasToggleSizeIcon : 1;
    unsigned wfHasVerticalScrollbar : 1;
    unsigned wfHasAdjustSizeIcon : 1;
    unsigned wfHasHorizontalScrollbar : 1;
    unsigned wfUsesNewBits : 1;
  } sWindowFlags; // wf

  typedef struct
  {
    tLocation lVisibleMinimum;
    tLocation lVisibleMaximum;
    tLocation lScrollOffset;
    tWindowHandle whBehind;
    union
    {
      sWindowFlags wfFlags;
      unsigned int uiFlags;
    };
    struct
    {
      unsigned bTitleForegroundColour : 8;
      unsigned bTitleBackgroundColour : 8;
      unsigned bWorkAreaForegroundColour : 8;
      unsigned bWorkAreaBackgroundColour : 8;
      unsigned bScrollbarOuterColour : 8;
      unsigned bScrollbarInnerColour : 8;
      unsigned bTitleFocusBackgroundColour : 8;
      unsigned bReserved : 8;
    };
    tLocation lWorkareaMinimum;
    tLocation lWorkareaMaximum;
    sIconFlags ifTitleBarIconFlags;
    struct
    {
      unsigned : 12;
      unsigned uWorkareaButtonType : 4;
      unsigned : 16;
    };
    void* vpSpriteArea;
    struct
    {
      unsigned ucMinimumXSize : 16;
      unsigned ucMinimumYSize : 16;
    };
    uIconData idTitleData;
    unsigned int uiIconCount;
  } tWindowCreateBlock; // wcb

  /*! Creates a window using the underlying creation block structure
  \param kpwcbWindow The window creation block
  \return The window handle, or 0 if an error occurred
  */
  tWindowHandle wimp_createwindow(const tWindowCreateBlock* kpwcbWindow);

  /*! Deletes a window
  \param kwhHandle The window handle to delete
  \return 0 if an error occurred
  */
  int wimp_deletewindow(const tWindowHandle kwhHandle);
  
  /*! Opens a window using the given open block
  \param kpwobOpenBlock The open block structure
  \return 0 if an error occurred
  */
  int wimp_openwindow(const tWindowOpenBlock* kpwobOpenBlock);

  /*! Closes a window
  \param kwhHandle The window handle to close
  \return 0 if an error occurred
  */
  int wimp_closewindow(const tWindowHandle kwhHandle);

#ifdef __cplusplus
}
#endif
